@@ -1,15 +1,11 @@
-from collections import Mapping
 import copy
-from inspect import isclass
 import json
+from collections import Mapping
+from inspect import isclass
 
-from .base import Object, Environment
-from .naming import Namable, Ref, frozendict
-from .typing import (
-  Type,
-  TypeCheck,
-  TypeFactory,
-  TypeMetaclass)
+from .base import Environment, Object
+from .naming import Namable, frozendict
+from .typing import Type, TypeCheck, TypeFactory, TypeMetaclass
 
 
 class Empty(object):
@@ -23,6 +19,13 @@ class TypeSignature(object):
   """
 
   def __init__(self, cls, required=False, default=Empty):
+    """Create an instance of a type signature.
+    Args:
+        cls (Class): the "type" of the object this signature represents.
+        required (bool):
+        default(object): an instance of the type for a default value. This
+           should be either an instance of cls or something coercable to cls.
+    """
     assert isclass(cls)
     assert issubclass(cls, Object)
     if default is not Empty and not isinstance(default, cls):
@@ -47,6 +50,13 @@ class TypeSignature(object):
     else:
       return TypeSignature(real_class, required=req)
 
+  def __hash__(self):
+    return hash(
+      self.klazz.serialize_type(),
+      self.required,
+      self.default,
+      self.empty)
+
   def __eq__(self, other):
     return (self.klazz.serialize_type() == other.klazz.serialize_type() and
             self.required == other.required and
@@ -78,6 +88,7 @@ class TypeSignature(object):
 
   @staticmethod
   def wrap(sig):
+    """Convert a Python class into a type signature."""
     if isclass(sig) and issubclass(sig, Object):
       return TypeSignature(sig)
     elif isinstance(sig, TypeSignature):
@@ -187,6 +198,9 @@ class Structural(Object, Type, Namable):
     new_self._update_schema_data(**copy.copy(kw))
     return new_self
 
+  def __hash__(self):
+    return hash(self.get())
+
   def __eq__(self, other):
     if not isinstance(other, Structural): return False
     if self.TYPEMAP != other.TYPEMAP: return False
@@ -216,11 +230,12 @@ class Structural(Object, Type, Namable):
     return lambda: self.interpolate_key(attr)
 
   def check(self):
+    scopes = self.scopes()
     for name, signature in self.TYPEMAP.items():
       if self._schema_data[name] is Empty and signature.required:
         return TypeCheck.failure('%s[%s] is required.' % (self.__class__.__name__, name))
       elif self._schema_data[name] is not Empty:
-        type_check = self._schema_data[name].in_scope(*self.scopes()).check()
+        type_check = self._schema_data[name].in_scope(*scopes).check()
         if type_check.ok():
           continue
         else:
@@ -273,8 +288,15 @@ class Structural(Object, Type, Namable):
 
   @classmethod
   def _filter_against_schema(cls, values):
-    return dict((key, val) for (key, val) in values.items()
-                if key in cls.TYPEMAP)
+    result = {}
+    for key, val in values.items():
+      if key not in cls.TYPEMAP:
+        continue
+      if issubclass(cls.TYPEMAP[key].klazz, Structural):
+        result[key] = cls.TYPEMAP[key].klazz._filter_against_schema(val)
+      else:
+        result[key] = val
+    return result
 
   @classmethod
   def json_load(cls, fp, strict=False):
