@@ -14,6 +14,9 @@ class frozendict(dict):
   def __eq__(self, other):
     return self.__key() == other.__key()
 
+  def __ne__(self, other):
+    return self.__key() != other.__key()
+
   def __repr__(self):
     return 'frozendict(%s)' % dict.__repr__(self)
 
@@ -56,7 +59,7 @@ class Ref(object):
   # ref re
   # ^[^\d\W]\w*\Z
   _DEREF_RE = r'[^\d\W]\w*'
-  _INDEX_RE = r'[\w\-]+'
+  _INDEX_RE = r'[\w\-\./]+'
   _REF_RE = re.compile(r'(\.' + _DEREF_RE + r'|\[' + _INDEX_RE + r'\])')
   _VALID_START = re.compile(r'[a-zA-Z_]')
   _COMPONENT_SEPARATOR = '.'
@@ -69,6 +72,9 @@ class Ref(object):
     def value(self):
       return self._value
 
+    def __hash__(self):
+      return hash(self.value)
+
     def __eq__(self, other):
       return self.__class__ == other.__class__ and self.value == other.value
 
@@ -82,7 +88,7 @@ class Ref(object):
       return self.value > other.value
 
   class Index(Component):
-    RE = re.compile('^[\w\-]+$')
+    RE = re.compile('^[\w\-\./]+$')
 
     def __repr__(self):
       return '[%s]' % self._value
@@ -118,7 +124,7 @@ class Ref(object):
     return Ref(components)
 
   def __init__(self, components):
-    self._components = components
+    self._components = tuple(components)
 
   def components(self):
     return self._components
@@ -186,6 +192,9 @@ class Ref(object):
   def __eq__(self, other):
     return self.components() == other.components()
 
+  def __ne__(self, other):
+    return self.components() != other.components()
+
   @staticmethod
   def compare(self, other):
     if len(self.components()) < len(other.components()):
@@ -202,4 +211,4 @@ class Ref(object):
     return Ref.compare(self, other) == 1
 
   def __hash__(self):
-    return hash(str(self))
+    return hash(self.components())
